

// CI01 - OCTAL CURRENT MONITOR


// Includes
#include <vector>
#include "device.hpp"


// Preprocessor Directives
#pragma once


// Class CI01
class CI01 : public Device
{
public:

   // Class Calibration
   class Calibration;

   // Class Exception
   class Exception;

   // Channels
   static const uint8_t CHANNELS = 8;

   // Sampling Frequencies [Hz]
   static const uint16_t FREQUENCY_6 = 6;
   static const uint16_t FREQUENCY_12 = 12;
   static const uint16_t FREQUENCY_25 = 25;
   static const uint16_t FREQUENCY_50 = 50;
   static const uint16_t FREQUENCY_100 = 100;
   static const uint16_t FREQUENCY_200 = 200;
   static const uint16_t FREQUENCY_400 = 400;
   static const uint16_t FREQUENCY_800 = 800;
   static const uint16_t FREQUENCY_1500 = 1500;
   static const uint16_t FREQUENCY_3000 = 3000;

   // Constructor
   CI01(const IP::Address& address);
   CI01(const Device& device);

   // Calibrate
   void calibrate(void) const;
   void calibrate(const Calibration& calibration) const;

   // Set Sampling Frequency [Hz]
   void frequency(uint16_t frequency) const;

   // Measure Current [A]
   float measure(uint8_t channel) const;
   std::map<uint8_t, float> measure(void) const;
   std::map<uint8_t, float> measure(const std::set<uint8_t>& channel) const;
   std::vector<float> measure(uint8_t channel, uint32_t samples, float frequency) const;
   std::vector<std::map<uint8_t, float>> measure(const std::set<uint8_t>& channel, uint32_t samples,
      float frequency) const;

   // Stop Current Measurement or Calibration
   void stop(void) const;

private:

   // DCP Message Types
   static const uint16_t _MESSAGE_MEASURE = 0x0400;
   static const uint16_t _MESSAGE_CURRENT = 0x0401;
   static const uint16_t _MESSAGE_ERROR_RESERVED = 0x0402;
   static const uint16_t _MESSAGE_ERROR_CONFIGURATION = 0x0403;
   static const uint16_t _MESSAGE_STOP = 0x0404;
   static const uint16_t _MESSAGE_ERROR_ABORTED = 0x0405;
   static const uint16_t _MESSAGE_SET_CALIBRATION = 0x0406;
   static const uint16_t _MESSAGE_BUSY = 0x0407;
   static const uint16_t _MESSAGE_SET_FREQUENCY = 0x0408;

   // Limits
   static const float _FREQUENCY_MIN;
   static const float _FREQUENCY_MAX;
   static const uint32_t _SAMPLES_MAX = 1000000L;

   // Calibration Expiration [Days]
   static const uint16_t _CALIBRATION_EXPIRATION = 365;
};


// Class Calibration
class CI01::Calibration
{
public:

   // Constructor
   Calibration(void);
   Calibration(const std::string& str);

   // Convert to String
   operator std::string(void) const;

   // Parse String
   Calibration& operator =(const std::string& str);

   // Current Coefficient List
   const std::vector<float>& current(uint8_t channel) const;
   void current(uint8_t channel, const std::vector<float>& current);

private:

   // Polynomial Degree
   static const uint8_t _DEGREE = 7;

   // Variables
   std::vector<std::vector<float>> _current;
};


// Class Exception
class CI01::Exception : public Device::Exception
{
public:

   // Class Aborted
   class Aborted;

   // Class Configuration
   class Configuration;

   // Class Reserved
   class Reserved;
};


// Class Aborted
class CI01::Exception::Aborted : public CI01::Exception
{
};


// Class Configuration
class CI01::Exception::Configuration : public CI01::Exception
{
};


// Class Reserved
class CI01::Exception::Reserved : public CI01::Exception
{
};
