

// VI01 - OCTAL VOLTAGE MONITOR


// Includes
#include <vector>
#include "device.hpp"


// Preprocessor Directives
#pragma once


// Class VI01
class VI01 : public Device
{
public:

   // Class Calibration
   class Calibration;

   // Class Exception
   class Exception;

   // Channels
   static const uint8_t CHANNELS = 8;

   // Sampling Frequencies [Hz]
   static const uint16_t FREQUENCY_6 = 6;
   static const uint16_t FREQUENCY_12 = 12;
   static const uint16_t FREQUENCY_25 = 25;
   static const uint16_t FREQUENCY_50 = 50;
   static const uint16_t FREQUENCY_100 = 100;
   static const uint16_t FREQUENCY_200 = 200;
   static const uint16_t FREQUENCY_400 = 400;
   static const uint16_t FREQUENCY_800 = 800;
   static const uint16_t FREQUENCY_1500 = 1500;
   static const uint16_t FREQUENCY_3000 = 3000;

   // Constructor
   VI01(const IP::Address& address);
   VI01(const Device& device);

   // Calibrate
   void calibrate(void) const;
   void calibrate(const Calibration& calibration) const;

   // Set Sampling Frequency [Hz]
   void frequency(uint16_t frequency) const;

   // Measure Voltage [V]
   float measure(uint8_t channel) const;
   std::map<uint8_t, float> measure(void) const;
   std::map<uint8_t, float> measure(const std::set<uint8_t>& channel) const;
   std::vector<float> measure(uint8_t channel, uint32_t samples, float frequency) const;
   std::vector<std::map<uint8_t, float>> measure(const std::set<uint8_t>& channel, uint32_t samples,
      float frequency) const;

   // Stop Voltage Measurement or Calibration
   void stop(void) const;

private:

   // DCP Message Types
   static const uint16_t _MESSAGE_MEASURE = 0x0900;
   static const uint16_t _MESSAGE_VOLTAGE = 0x0901;
   static const uint16_t _MESSAGE_ERROR_RESERVED = 0x0902;
   static const uint16_t _MESSAGE_ERROR_CONFIGURATION = 0x0903;
   static const uint16_t _MESSAGE_STOP = 0x0904;
   static const uint16_t _MESSAGE_ERROR_ABORTED = 0x0905;
   static const uint16_t _MESSAGE_SET_CALIBRATION = 0x0906;
   static const uint16_t _MESSAGE_BUSY = 0x0907;
   static const uint16_t _MESSAGE_SET_FREQUENCY = 0x0908;

   // Limits
   static const float _FREQUENCY_MIN;
   static const float _FREQUENCY_MAX;
   static const uint32_t _SAMPLES_MAX = 1000000L;

   // Calibration Expiration [Days]
   static const uint16_t _CALIBRATION_EXPIRATION = 365;
};


// Class Calibration
class VI01::Calibration
{
public:

   // Constructor
   Calibration(void);
   Calibration(const std::string& str);

   // Convert to String
   operator std::string(void) const;

   // Parse String
   Calibration& operator =(const std::string& str);

   // Voltage Coefficient List
   const std::vector<float>& voltage(void) const;
   void voltage(const std::vector<float>& voltage);

private:

   // Polynomial Degree
   static const uint8_t _DEGREE = 7;

   // Variables
   std::vector<float> _voltage;
};


// Class Exception
class VI01::Exception : public Device::Exception
{
public:

   // Class Aborted
   class Aborted;

   // Class Configuration
   class Configuration;

   // Class Reserved
   class Reserved;
};


// Class Aborted
class VI01::Exception::Aborted : public VI01::Exception
{
};


// Class Configuration
class VI01::Exception::Configuration : public VI01::Exception
{
};


// Class Reserved
class VI01::Exception::Reserved : public VI01::Exception
{
};
